@@ -24,8 +24,10 @@
 #include "downloadjob.h"
 #include "settings.h"
 
+#include <QDateTime>
 #include <QJsonObject>
 
+#include <KFileMetaData/UserMetaData>
 #include <KLocalizedString>
 
 #include <KIO/Global>
@@ -98,6 +100,8 @@ void DownloadJob::update(const QJsonObject &payload)
 
     it = payload.constFind(QStringLiteral("filename"));
     if (it != end) {
+        m_fileName = it->toString();
+
         const QUrl destination = QUrl::fromLocalFile(it->toString());
 
         setProperty("destUrl", destination.toString(QUrl::RemoveFilename | QUrl::StripTrailingSlash));
@@ -106,6 +110,16 @@ void DownloadJob::update(const QJsonObject &payload)
         descriptionDirty = true;
     }
 
+    it = payload.constFind(QStringLiteral("mime"));
+    if (it != end) {
+        m_mimeType = it->toString();
+    }
+
+    it = payload.constFind(QStringLiteral("incognito"));
+    if (it != end) {
+        m_incognito = it->toBool();
+    }
+
     it = payload.constFind(QStringLiteral("totalBytes"));
     if (it != end) {
         const qlonglong totalAmount = it->toDouble();
@@ -222,6 +236,10 @@ void DownloadJob::update(const QJsonObject &payload)
         if (state == QLatin1String("complete")) {
             setError(KJob::NoError);
             setProcessedAmount(KJob::Files, 1);
+
+            // Write origin url into extended file attributes
+            saveOriginUrl();
+
             emitResult();
             return;
         }
@@ -235,3 +253,26 @@ void DownloadJob::updateDescription()
         qMakePair<QString, QString>(i18nc("The location being downloaded to", "Destination"), m_destination.toLocalFile())
     );
 }
+
+void DownloadJob::saveOriginUrl()
+{
+    if (m_incognito
+        // Blob URLs are dynamically created through JavaScript and cannot be accessed from the outside
+        || m_finalUrl.scheme() == QLatin1String("blob")) {
+        return;
+    }
+
+    const QJsonObject settings = Settings::self().settingsForPlugin(QStringLiteral("downloads"));
+
+    const bool saveOriginUrl = settings.value(QStringLiteral("saveOriginUrl")).toBool();
+    if (!saveOriginUrl) {
+        return;
+    }
+
+    KFileMetaData::UserMetaData md(m_fileName);
+
+    QUrl url = m_finalUrl;
+    url.setPassword(QString());
+
+    md.setOriginUrl(url);
+}
