@@ -30,10 +30,17 @@
 #include <KJobTrackerInterface>
 
 DownloadPlugin::DownloadPlugin(QObject* parent) :
-    AbstractBrowserPlugin(QStringLiteral("downloads"), 1, parent)
+    AbstractBrowserPlugin(QStringLiteral("downloads"), 2, parent)
 {
 }
 
+bool DownloadPlugin::onLoad()
+{
+    // Have extension tell us about all the downloads
+    sendData(QStringLiteral("createAll"));
+    return true;
+}
+
 bool DownloadPlugin::onUnload()
 {
     for (auto it = m_jobs.constBegin(), end = m_jobs.constEnd(); it != end; ++it) {
